﻿using System;
using System.Collections.Generic;
using Caliburn.Micro;
using ESRI.ArcGIS.Client.Tasks;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.DataContracts;
using SpatialReference = ESRI.ArcGIS.Client.Geometry.SpatialReference;

namespace FDOT.GIS.Client.Components.ViewModels.AddressLookupTool
{
    public class AddressLocator : ResultBase
    {
        private readonly string _city;
        private readonly string _streetAddress;
        private readonly string _zipCode;

        public AddressLocator(string streetAddress, string city, string state, string zipCode)
        {
            _streetAddress = streetAddress;
            _city = city;
            _zipCode = zipCode;
        }

        public AddressLocatorResult Result { get; private set; }

        public override void Execute(ActionExecutionContext context)
        {
            var locatorTask =
                new Locator("http://tasks.arcgisonline.com/ArcGIS/rest/services/Locators/TA_Streets_US/GeocodeServer");
            locatorTask.AddressToLocationsCompleted += LocatorTaskAddressToLocationsCompleted;
            locatorTask.Failed += LocatorTaskFailed;
            var addressParams = new AddressToLocationsParameters
                                    {
                                        OutSpatialReference =
                                            new SpatialReference(Convert.ToInt32(MapHandler.MapSpatialReference.WellKnownId))
                                    };
            Dictionary<string, string> address = addressParams.Address;
            addressParams.OutFields.Add("Side");

            if (!string.IsNullOrEmpty(_streetAddress))
                address.Add("Street", _streetAddress);
            if (!string.IsNullOrEmpty(_city))
                address.Add("City", _city);

            address.Add("State", "FL");
            if (!string.IsNullOrEmpty(_zipCode))
                address.Add("ZIP", _zipCode);

            locatorTask.AddressToLocationsAsync(addressParams);
        }

        private void LocatorTaskFailed(object sender, TaskFailedEventArgs e)
        {
            Failure(e.Error);
        }

        private void LocatorTaskAddressToLocationsCompleted(object sender, AddressToLocationsEventArgs e)
        {
            var addressAttribute = new AttributeDescription("Address", "Address", typeof(string), true, true);
            var scoreAttribute = new AttributeDescription("Score", "Score", typeof(string), true, false);
            var sideAttribute = new AttributeDescription("Side", "Side of Street", typeof(string), true, false);

            var attributeDescriptions = new List<AttributeDescription> { addressAttribute, scoreAttribute, sideAttribute };

            var featureType = new FeatureType("address", "Address Information", "Geocoded address record", true, DataContracts.Enums.GeometryTypes.Point, attributeDescriptions);

            var features = new List<Feature>();

            foreach (AddressCandidate r in e.Results)
            {
                //TODO: Add code to reproject into other projections besides WebMercator
                //var webMercatorProjection = new ESRI.ArcGIS.Client.Projection.WebMercator();
                //var projectedGeometry = webMercatorProjection.FromGeographic(r.Location);

                var dataDict = new Dictionary<AttributeDescription, object>
                                   {
                                       { addressAttribute, r.Address },
                                       { scoreAttribute, r.Score },
                                       { sideAttribute, r.Attributes["Side"] },
                                   };

                var geometry = new DataContracts.Geometry
                                   {
                                       X = r.Location.X,
                                       Y = r.Location.Y
                                   };

                var feature = new Feature(featureType, dataDict) { Geometry = geometry };

                features.Add(feature);
            }

            Result = new AddressLocatorResult(featureType, features);
            Success();
        }
    }

    public class AddressLocatorResult
    {
        public AddressLocatorResult(FeatureType featureType, ICollection<Feature> features)
        {
            FeatureType = featureType;
            Features = features;
        }

        public FeatureType FeatureType { get; private set; }
        public ICollection<Feature> Features { get; private set; }
    }
}