﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Components.Views.AddressLookupTool;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.AddressLookupTool
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class AddressLookupViewModel : ComponentBase
    {
        private bool _isBusy;
        private bool _isValidSearchAddress;

        public AddressLookupViewModel()
        {
            StreetAddress = string.Empty;
            City = string.Empty;
            ZipCode = string.Empty;
        }

        public string StreetAddress { get; set; }
        public string City { get; set; }
        public string ZipCode { get; set; }
        private string _locatedStreetAddress;
        public string LocatedStreetAddress
        {
            get { return _locatedStreetAddress; }
            set { _locatedStreetAddress = value; NotifyOfPropertyChange(() => LocatedStreetAddress); }
        }

        private string _locatedCity;
        public string LocatedCity
        {
            get { return _locatedCity; }
            set { _locatedCity = value; NotifyOfPropertyChange(() => LocatedCity); }
        }

        private string _locatedZipCode;
        public string LocatedZipCode
        {
            get { return _locatedZipCode; }
            set { _locatedZipCode = value; NotifyOfPropertyChange(() => LocatedZipCode); }
        }

        protected GenericDialogViewModel Dialog { get; set; }

        public override DependencyObject HelpView
        {
            get
            {
                var help = new AddressLookupHelpView();
                View.SetModel(help, new AddressLookupHelpViewModel());
                return help;
            }
        }

        public bool IsBusy
        {
            get { return _isBusy; }
            set
            {
                _isBusy = value;
                NotifyOfPropertyChange(() => IsBusy);
            }
        }

        public bool IsValidSearchAddress
        {
            get { return _isValidSearchAddress; }
            set
            {
                _isValidSearchAddress = value;
                NotifyOfPropertyChange(() => IsValidSearchAddress);
            }
        }

        public void LocateAddress()
        {
            var parameters = new GeometrySelectionServiceConfiguration { Color = Colors.Red };
            var geometrySelectionService = new GeometrySelectionService(parameters);

            geometrySelectionService.SelectGeometry(GeometryType.Point, GeometrySelected);
        }

        private void GeometrySelected(DataContracts.Geometry obj)
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(LocateAddressWorkflow(obj).GetEnumerator(), executionContext, Complete);
        }

        protected override void OnExecute()
        {
            var view = new AddressLookupView();
            View.SetModel(view, this);
            Dialog = LayoutManager.Instance.ShowPanel(view,
                                                      new DialogServiceArgs
                                                          {
                                                              Title = ComponentConfiguration.Name,
                                                              HelpId = "tools;" + GetType().FullName,
                                                              Icon = Icon
                                                          });
        }

        public void Find()
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetAddressFindWorkflow().GetEnumerator(), executionContext, Complete);
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private IEnumerable<IResult> LocateAddressWorkflow(DataContracts.Geometry geometry)
        {
            IsBusy = true;
            var getProjection = new GetProjection(geometry, WellKnownIds.WGS1984);
            getProjection.Completed += (s1, e1) =>
                                           {
                                               var locate = new GetLocationAddress(getProjection.Result);
                                               locate.Completed += (s2, e2) =>
                                                                       {
                                                                           if (locate.Result != null)
                                                                           {
                                                                               LocatedStreetAddress = locate.Result["Street"].ToString();
                                                                               LocatedCity = locate.Result["City"].ToString();
                                                                               LocatedZipCode = locate.Result["ZIP"].ToString();
                                                                           }
                                                                           else
                                                                           {
                                                                               LocatedStreetAddress = string.Empty;
                                                                               LocatedCity = string.Empty;
                                                                               LocatedZipCode = string.Empty;
                                                                               LayoutManager.Instance.ShowMessage(
                                                                                   "An address was not found for the selected location.",
                                                                                   new TimeSpan(0, 0, 0, 8));
                                                                           }
                                                                           IsBusy = false;
                                                                       };
                                               locate.Execute(new ActionExecutionContext());
                                           };

            yield return getProjection;
        }

        private IEnumerable<IResult> GetAddressFindWorkflow()
        {
            IsBusy = true;
            var addressLocator = new GetAddressLocations(StreetAddress, City, "FL", ZipCode);
            yield return addressLocator;
            IsBusy = false;

            if (addressLocator.Result != null)
            {
                FeatureType featureType = addressLocator.Result.FeatureType;
                ICollection<Feature> features = addressLocator.Result.Features;
                int resultCount = features.Count();

                if (resultCount == 0)
                {
                    LayoutManager.Instance.ShowMessage("No results found for this address!", new TimeSpan(0, 0, 0, 8));
                    yield break;
                }

                var queryResult = new QueryResult(featureType, new Core.ObservableCollection<Feature>(features));
                var args = new ResultDisplayArgs(queryResult) { Title = "Address Results" };
                ResultDisplayManager.Instance.DisplayResults(args);

                if (resultCount == 1 && features.First().Geometry.CurrentGeometryType == DataContracts.Enums.GeometryTypes.Point)
                { MapHandler.ZoomToGeometry(features.First().Geometry); }
                else
                {
                    var zoomTo = new ZoomToGeometry(features, 2);
                    yield return zoomTo;
                }
            }
            else
            { LayoutManager.Instance.ShowMessage("No results found for this address!"); }
            yield break;
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/address_lookup.png"; }
        }

        public void ValidateAddressSearchCriteria()
        {
            IsValidSearchAddress = !string.IsNullOrEmpty(StreetAddress);
        }

        //private string addressField;
        //private string cityField;
        //private string zipField;
        //private string parcelMapserviceName;
        private readonly List<int> _parcelLayerIds = new List<int>();

        private static bool IsConfiguredForParcelLookup(IDictionary<string, string> customSettings)
        {
            return customSettings.Keys.Contains("addressFieldName")
                && customSettings.Keys.Contains("cityFieldName")
                && customSettings.Keys.Contains("zipFieldName")
                && customSettings.Keys.Contains("parcelLayersMapService")
                && customSettings.Keys.Contains("parcelLayers");
        }

        protected override void OnInitialize(IDictionary<string, string> customSettings)
        {
            if (!IsConfiguredForParcelLookup(customSettings))
            {
                return;
            }

            //addressField = customSettings["addressFieldName"];
            //cityField = customSettings["cityFieldName"];
            //zipField = customSettings["zipFieldName"];

            //string msName = customSettings["parcelLayersMapService"];
            //if (VirtualApplicationManager.Current.MapServices.Any(ms => ms.Name.Equals(msName, StringComparison.OrdinalIgnoreCase)))
            //    parcelMapserviceName = msName;

            string[] temp = customSettings["parcelLayers"].Split(new[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            foreach (var parcelLayerId in temp)
            {
                if (parcelLayerId.Contains("-"))
                {
                    string[] parts = parcelLayerId.Split(new[] {'-'}, StringSplitOptions.RemoveEmptyEntries);
                    if (parts.Length != 2) continue;

                    int first, second;

                    if (!int.TryParse(parts[0], out first)) continue;
                    if (!int.TryParse(parts[1], out second)) continue;

                    if (first == second)
                    {
                        _parcelLayerIds.Add(first);
                    }
                    else if (first < second)
                    {
                        _parcelLayerIds.AddRange(Enumerable.Range(first, second - first));
                    }
                    else
                    {
                        _parcelLayerIds.AddRange(Enumerable.Range(second, first - second));
                    }
                }
                else
                {
                    int parcelId;
                    if (!int.TryParse(parcelLayerId, out parcelId)) continue;
                    _parcelLayerIds.Add(parcelId);
                }
            }
        }
    }

}