﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.ViewModels;
using FDOT.GIS.DataContracts;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class CoordinatesViewModel : ComponentBase
    {
        private readonly GeometrySelectionService _geometrySelectionService = new GeometrySelectionService(new GeometrySelectionServiceConfiguration());
        private GraphicsManager _graphicsManager;
        private readonly List<Tuple<string, string>> _projections = new List<Tuple<string, string>>();
        private GenericDialogViewModel _dialogViewModel;
        private double _y;
        private double _x;
        private string _wkid;

        private bool _projectionVisibility;

        #region Properties

        private bool _isBusy;
        public bool IsBusy
        {
            get { return _isBusy; }
            set 
            { 
                _isBusy = value; 
                NotifyOfPropertyChange(() => IsBusy); 
            }
        }

        public bool ProjectionVisibility
        {
            get { return _projectionVisibility; }
            set
            {
                _projectionVisibility = value;
                NotifyOfPropertyChange(() => ProjectionVisibility);
            }
        }

        public override DependencyObject HelpView
        {
            get
            {
                var help = new CoordinatesHelpView();
                View.SetModel(help, new CoordinatesHelpViewModel { SupportedProjections = _projections.Select(t => t.Item2).ToList() });
                return help;
            }
        }

        public double Y
        {
            get { return Math.Round(_y,5); }
            set
            {
                _y = value;
                NotifyOfPropertyChange(() => Y);
            }
        }

        public Tuple<string, string> SelectedValue
        {
            get { return _projections.First(); }
        }

        public List<Tuple<string, string>> Projections
        {
            get { return _projections; }
        }

        public double X
        {
            get { return Math.Round(_x,5); }
            set
            {
                _x = value;
                NotifyOfPropertyChange(() => X);
            }
        }
        #endregion

        #region Methods
        private void SetProjections()
        {

            if (Projections.Count > 0)
                return;

            _projections.Add(new Tuple<string, string>(WellKnownIds.WGS1984, "WGS-1984"));

            if (ComponentConfiguration.CustomSettings == null ||
                !ComponentConfiguration.CustomSettings.ContainsKey("Projections")) return;

            foreach (var splitItems in
                ComponentConfiguration.CustomSettings["Projections"].Split(',').Select(item => item.Split(':')))
            {
                _projections.Add(new Tuple<string, string>(splitItems[0], splitItems[1]));
            }
        }

        protected override void OnInitialize(IDictionary<string, string> customSettings)
        {
            SetProjections();
        }

        protected override void OnExecute()
        {
            var view = new CoordinatesView();
            View.SetModel(view, this);
            _dialogViewModel = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs { Title = "Get Coordinates", HelpId = "tools;" + GetType().FullName, Icon = Icon });
            _dialogViewModel.Closed += DialogViewModelClosed;

            if (MapHandler.MapSpatialReference.WellKnownId == WellKnownIds.WebMercator)
            { MapHandler.MouseMove += CurrentMapMouseMove; }

            if (_graphicsManager != null)
                _graphicsManager.Dispose();

            _graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
            _geometrySelectionService.SelectGeometry(GeometryType.Point, GetCoordinates,
                                                     "Select a point to get it's coordinates");
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/get_coordinates.png"; }
        }

        internal void DialogViewModelClosed(object sender, EventArgs e)
        {
            if (MapHandler.MapSpatialReference.WellKnownId == WellKnownIds.WebMercator)
            { MapHandler.MouseMove -= CurrentMapMouseMove; }

            _graphicsManager.Dispose();
            _geometrySelectionService.CancelSelection();
        }

        private void CurrentMapMouseMove(object sender, MouseEventArgs e)
        {
            CalculateLatLong(MapHandler.GetMouseCoordinates(e));
        }

        public void GetCoordinates(DataContracts.Geometry geometry)
        {
            if (MapHandler.MapSpatialReference.WellKnownId == WellKnownIds.WebMercator)
            {
                MapHandler.MouseMove -= CurrentMapMouseMove;
                CalculateLatLong(geometry);
            }
            else
            {
                _wkid = MapHandler.MapSpatialReference.WellKnownId;
                ChangeCoordinates(Projections.First());
            }

            _graphicsManager.AddGraphic(geometry, new SymbolDefinition("RotatingBlueMarkerSymbol"), null, null);

            ProjectionVisibility = true;
        }

        public void ChangeCoordinates(Tuple<string, string> projection)
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetWorkflow(projection.Item1).GetEnumerator(), executionContext, Complete);

        }

        private IEnumerable<IResult> GetWorkflow(string outWkid)
        {
            IsBusy = true;
            var project = new Project(new SpatialReference { WellKnownId = _wkid },
                                      new SpatialReference { WellKnownId = outWkid },
                                      new DataContracts.Geometry { X = X, Y = Y });

            yield return project;

            DataContracts.Geometry geometry = project.Result.First();
            if (geometry.X != null) X = (double)geometry.X;
            if (geometry.Y != null) Y = (double)geometry.Y;
            _wkid = outWkid;

            IsBusy = false;
            yield break;
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private void CalculateLatLong(DataContracts.Geometry geometry)
        {
            DataContracts.Geometry point = GeometryHelper.CalculateLatLong(geometry);
            Y = point.Y ?? 0;
            X = point.X ?? 0;
            _wkid = WellKnownIds.WGS1984;
        }

        //private void CalculateLatLong(DataContracts.Geometry geometry)
        //{
        //    ESRI.ArcGIS.Client.Geometry.Geometry translatedGeo = GeometryTranslator.Translate(geometry);
        //    CalculateLatLong((MapPoint)translatedGeo);
        //}
        #endregion
    }
}