﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Windows;
using System.Windows.Media;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Controls;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Domain.Query;
using FDOT.GIS.Client.ImageManipulation;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [DisplayName("Draw")]
    public class DrawToolViewModel : ComponentBase
    {
        private IDictionary<string, string> _customSettings;
        private GeometrySelectionService _geometryService;
        private GenericDialogViewModel _dialogViewModel;
        private GraphicsManager _graphicsManager;
        private bool _isDrawing;

        public bool ShowLabels
        {
            get { return _graphicsManager.ShowLabels; }
            set { _graphicsManager.ShowLabels = value; }
        }


        public void Draw(ShapeSelectedEventArgs e)
        {
            if (_isDrawing)
                _geometryService.CancelSelection();

            _isDrawing = true;
            //Symbol symbol = GetSymbol(e.ShapeSelected);
            SymbolManager.SetPalette(CurrentDrawColor);
            _geometryService.SelectGeometry(e.ShapeSelected, g =>
            {
                _isDrawing = false;
                var f = new Feature(null, null, new FeatureEditOptions {CanDelete = true, CanEditGeometry = true})
                            {Geometry = g};
                //_graphicsManager.AddGraphic(g, null, null, null);
                _graphicsManager.AddGraphic(f, null, null);
                SymbolManager.SetDefaultPalette();
            });       
        }


        private Color _currentDrawColor = SymbolManager.DefaultColor;
        public string CurrentDrawColor
        {
            get { return ColorConverter.ColorHex(_currentDrawColor); }
            set
            { 
                _currentDrawColor = ColorConverter.HexColor(value);
                NotifyOfPropertyChange(() => CurrentDrawColor);
            }
        }

        public void DrawColorChanged(ColorChangedEventArgs e)
        { _currentDrawColor = e.NewColor; }


        //private Symbol GetSymbol(GeometryType geometryType)
        //{
        //    switch (geometryType)
        //    {
        //        case GeometryType.Line:             return SymbolManager.GetLineSymbol(_currentDrawColor);
        //        case GeometryType.GeodesicLine:     return SymbolManager.GetLineSymbol(_currentDrawColor);
        //        case GeometryType.Polyline:         return SymbolManager.GetLineSymbol(_currentDrawColor);
        //        case GeometryType.Point: return SymbolManager.GetLineSymbol(_currentDrawColor);
        //        default: return SymbolManager.GetFillSymbol(_currentDrawColor);
        //    }
        //}



        public void Clear()
        {
            _graphicsManager.ClearGraphics();
            _geometryService.CancelSelection();
            CurrentDrawColor = ColorConverter.ColorHex(SymbolManager.DefaultColor);
        }


        public void LabelVisibilityChanged(object sender, EventArgs e)
        {
            NotifyOfPropertyChange(() => ShowLabels);
        }


        #region component implementation

        public override DependencyObject HelpView
        {
            get
            {
                var help = new DrawHelpView();
                View.SetModel(help, new DrawHelpViewModel());
                return help;
            }
        }

        protected override void OnInitialize(IDictionary<string, string> customSettings)
        {
            _graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
            _graphicsManager.LabelVisibilityChanged += LabelVisibilityChanged;

            _customSettings = customSettings;
            _geometryService = new GeometrySelectionService(new GeometrySelectionServiceConfiguration { ShowLabels=true, Color = Colors.Green });
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/draw.png"; }
        }

        protected override void OnExecute()
        {
            var drawView = new DrawToolView();
            View.SetModel(drawView, this);

            #region load/set a custom property to persist graphic label visibility preference

            if (!_customSettings.ContainsKey("ShowLabels"))
            {
                _customSettings.Add("ShowLabels", true.ToString());
            }
            bool doShowLabels;
            ShowLabels = !bool.TryParse(_customSettings["ShowLabels"], out doShowLabels) || doShowLabels;

            #endregion

            _dialogViewModel = LayoutManager.Instance.ShowPanel(drawView,
                                                      new DialogServiceArgs
                                                      {
                                                          Title = ComponentConfiguration.Name,
                                                          HelpId = "tools;" + GetType().FullName,
                                                          Icon = Icon
                                                      });           
            _dialogViewModel.Closed += OnClose;
        }


        private void OnClose(object sender, EventArgs e)
        {
            // persist graphic label visibility preference
            _customSettings["ShowLabels"] = (_graphicsManager.ShowLabels && GraphicsManager.CanShowLabels).ToString();

            if (_graphicsManager == null) return;
            _graphicsManager.ClearGraphics();
            _dialogViewModel.Closed -= OnClose;
            _dialogViewModel = null;
            
            Clear();            
        }

        protected override void OnError(ErrorEventArgs errorEventArgs)
        {
            MessageBox.Show(errorEventArgs.Exception.ToString());
            errorEventArgs.Handled = true;
        }

        #endregion
    }


    #region moved into its own file and renamed DrawToolViewModel
    //[DisplayName("Draw")]
    //public class DrawComponentView : INotifyPropertyChanged
    //{
    //    #region DrawTypes enum

    //    public enum DrawTypes
    //    {
    //        Line,
    //        Box,
    //        Polygon,
    //        Polyline,
    //        Circle
    //    }

    //    #endregion

    //    private readonly GraphicsManager gm;
    //    private readonly GeometrySelectionService gss;
    //    private bool _isDrawing;

    //    public DrawComponentView(GraphicsManager gm, GeometrySelectionService gss)
    //    {
    //        this.gm = gm;
    //        this.gss = gss;
    //        gm.LabelVisibilityChanged += LabelVisibilityChanged;
    //    }

    //    [Dropdown("DrawTypes")]
    //    public DrawTypes DrawType { get; set; }


    //    [Check]
    //    public bool ShowLabels
    //    {
    //        get { return gm.ShowLabels; }
    //        set { gm.ShowLabels = value; }
    //    }


    //    public void Draw()
    //    {
    //        if (_isDrawing)
    //            gss.CancelSelection();

    //        _isDrawing = true;
    //        gss.SelectGeometry(TranslateDrawType(DrawType), g =>
    //                                                            {
    //                                                                _isDrawing = false;
    //                                                                gm.AddGraphic(g, null, null, null);
    //                                                            });
    //    }

    //    public void Clear()
    //    {
    //        gm.ClearGraphics();
    //        gss.CancelSelection();
    //    }

    //    private static GeometryType TranslateDrawType(DrawTypes drawType)
    //    {
    //        switch (drawType)
    //        {
    //            case DrawTypes.Line:
    //                return GeometryType.Line;
    //            case DrawTypes.Polyline:
    //                return GeometryType.Polyline;
    //            case DrawTypes.Box:
    //                return GeometryType.Box;
    //            case DrawTypes.Polygon:
    //                return GeometryType.Polygon;
    //            case DrawTypes.Circle:
    //                return GeometryType.Circle;
    //            default:
    //                throw new ArgumentOutOfRangeException("drawType");
    //        }
    //    }


    //    public void LabelVisibilityChanged(object sender, EventArgs e)
    //    {
    //        NotifyPropertyChanged("ShowLabels");
    //    }

    //    #region INotifyPropertyChanged implementation

    //    public event PropertyChangedEventHandler PropertyChanged;

    //    private void NotifyPropertyChanged(String info)
    //    {
    //        if (PropertyChanged != null)
    //        {
    //            PropertyChanged(this, new PropertyChangedEventArgs(info));
    //        }
    //    }

    //    #endregion
    //}

    //[Export(typeof(IComponentBase))]
    //[PartCreationPolicy(CreationPolicy.Shared)]
    //public class TestQueryResultsComponent : ComponentBase
    //{
    //    protected override void OnExecute()
    //    {
    //        var l = new List<MainViewModel.TestResults>();
    //        for (int i = 0; i < 10; i++)
    //        {
    //            l.Add(new MainViewModel.TestResults { Id = i, Name = "Result " + i, RoadwayId = "55070000", ContractId = "A1001" });
    //        }
    //        var args = new ResultDisplayArgs();
    //        ResultDisplayManager.Instance.DisplayResults(l, i => i.Name, args);
    //    }

    //}

    //    protected override string GetIconResourceUri()
    //    {
    //        return "Assets/Images/measure.png";
    //    }
    //}
    #endregion

}