﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.Views.MeasureTool;
using FDOT.GIS.Client.Controls;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Drawing;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Domain.Measurement;
using FDOT.GIS.Client.Domain.UnitConversion;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.MeasureTool
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [DisplayName("Measure")]
    public class MeasureToolViewModel : ComponentBase
    {
        #region Private Properties
        private bool _isDrawing;
        private GeometryType _type;
        private double _area;
        private double _circumference;
        private double _diameter;
        private double _length;
        private double _perimeter;
        private double _radius;
        private bool _showArea;
        private bool _showCircumference;
        private bool _showDiameter;
        private bool _showLength;
        private bool _showPerimeter;
        private bool _showRadius;
        private DataContracts.Geometry _geometry;
        //private double radius;
        protected GenericDialogViewModel Dialog { get; set; }
        #endregion

        private readonly ILog _logger = LogManager.GetLog(typeof(MeasureToolViewModel));
        private static int _count;

        public MeasureToolViewModel()
        {
            PropertyChanged += MeasureToolViewModel_PropertyChanged;
            _logger.Debug("Constructor Invoked: {0}", ++_count);
        }


        public void ShapeSelected(ShapeSelectedEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(e.GetType().ToString());
            GoToResults(e.ShapeSelected);
        }


        void MeasureToolViewModel_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "SelectedUnit")
            {
                GeometryChange(_geometry, _radius);
            }
        }

        public ObservableCollection<Unit> UnitCollection
        {
            get
            {
                return new ObservableCollection<Unit>
                    (new List<Unit> { Unit.Feet, Unit.Meter, Unit.Mile, Unit.Kilometer, Unit.Acre });
            }
        }

        private Unit? _selectedUnit = Unit.Mile;

        public Unit SelectedUnit
        {
            get { return _selectedUnit.HasValue ? _selectedUnit.Value : UnitCollection.First(); }
            set
            {
                _selectedUnit = value;
                NotifyOfPropertyChange(() => SelectedUnit);
                NotifyOfPropertyChange(() => Units);
                NotifyOfPropertyChange(() => ShowAreaSquared);

                FilterFields();
            }
        }

        private Unit _circleStartUnit;

        private void FilterFields()
        {
            bool showLinearMeasurements = _selectedUnit != Unit.Acre;
            bool hasArea = _type == GeometryType.Box || _type == GeometryType.Polygon || _type == GeometryType.Circle;
            bool isCircle = _type == GeometryType.Circle;

            ShowLength = showLinearMeasurements && !hasArea;
            ShowPerimeter = showLinearMeasurements && hasArea && !isCircle;
            ShowArea = hasArea;
            ShowRadius = isCircle && showLinearMeasurements;
            ShowDiameter = isCircle && showLinearMeasurements;
            ShowCircumference = isCircle && showLinearMeasurements;
        }

        public bool ShowArea
        {
            get { return _showArea; }
            private set
            {
                _showArea = value;
                NotifyOfPropertyChange(() => ShowArea);
                NotifyOfPropertyChange(() => ShowAreaSquared);
            }
        }

        public bool ShowAreaSquared
        {
            get
            {
                return SelectedUnit != Unit.Acre;
            }
        }

        public bool ShowPerimeter
        {
            get { return _showPerimeter; }
            private set
            {
                _showPerimeter = value;
                NotifyOfPropertyChange(() => ShowPerimeter);
            }
        }

        public bool ShowDiameter
        {
            get { return _showDiameter; }
            private set
            {
                _showDiameter = value;
                NotifyOfPropertyChange(() => ShowDiameter);
            }
        }

        public bool ShowRadius
        {
            get { return _showRadius; }
            private set
            {
                _showRadius = value;
                NotifyOfPropertyChange(() => ShowRadius);
            }
        }

        public bool ShowCircumference
        {
            get { return _showCircumference; }
            private set
            {
                _showCircumference = value;
                NotifyOfPropertyChange(() => ShowCircumference);
            }
        }

        public bool ShowLength
        {
            get { return _showLength; }
            private set
            {
                _showLength = value;
                NotifyOfPropertyChange(() => ShowLength);
            }
        }

        public double Area
        {
            get { return _area; }
            private set
            {
                _area = value;
                NotifyOfPropertyChange(() => Area);
            }
        }

        public double Perimeter
        {
            get { return _perimeter; }
            private set
            {
                _perimeter = value;
                NotifyOfPropertyChange(() => Perimeter);
            }
        }

        public double Diameter
        {
            get { return _diameter; }
            private set
            {
                _diameter = value;
                NotifyOfPropertyChange(() => Diameter);
            }
        }

        public double Radius
        {
            get { return _radius; }
            private set
            {
                _radius = value;
                NotifyOfPropertyChange(() => Radius);
            }
        }

        public double Circumference
        {
            get { return _circumference; }
            private set
            {
                _circumference = value;
                NotifyOfPropertyChange(() => Circumference);
            }
        }

        public double Length
        {
            get { return _length; }
            private set
            {
                _length = value;
                NotifyOfPropertyChange(() => Length);
            }
        }

        public string Units
        {
            get { return UnitConverter.GetLabel(SelectedUnit); }
        }

        public Unit Unit
        {
            get { return SelectedUnit; }
        }

        private SquareUnit SquareUnit
        {
            get { return SelectedUnit == Unit.Acre ? SquareUnit.Acre : UnitConversion.GetSquareUnitFromUnit(SelectedUnit); }
        }

        private GraphicsManager _graphicsManager;
        public GraphicsManager GraphicsManager
        {
            get
            {
                return _graphicsManager;
            }
        }

        private GeometrySelectionService _geometryService;
        public GeometrySelectionService GeometryService
        {
            get
            {
                return _geometryService ??
                       (_geometryService =
                        new GeometrySelectionService(new GeometrySelectionServiceConfiguration { ShowLabels = true, Color = Colors.Green }));
            }
        }

        private void GoToResults(GeometryType measurementType)
        {
            ResetLabels();

            switch (measurementType)
            {
                case GeometryType.Box:
                    ShowArea = true;
                    ShowPerimeter = true;
                    _type = measurementType;
                    break;
                case GeometryType.Circle:
                    ShowArea = true;
                    ShowDiameter = true;
                    ShowRadius = true;
                    ShowCircumference = true;
                    _type = measurementType;
                    _circleStartUnit = Unit;
                    break;
                case GeometryType.Polyline:
                    ShowLength = true;
                    _type = measurementType;
                    break;
                case GeometryType.Polygon:
                    ShowArea = true;
                    ShowPerimeter = true;
                    _type = measurementType;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("measurementType");
            }

            FilterFields();

            if (GraphicsManager != null)
            {
                GraphicsManager.ClearGraphics();
            }
            StartSelection();
        }

        public void ResetLabels()
        {
            Area = 0;
            Perimeter = 0;
            Radius = 0;
            Circumference = 0;
            Diameter = 0;
            Length = 0;

            ShowArea = false;
            ShowCircumference = false;
            ShowDiameter = false;
            ShowLength = false;
            ShowPerimeter = false;
            ShowRadius = false;
        }

        public void StartSelection()
        {
            Unit unit = _type == GeometryType.Circle && Unit == Unit.Acre ? Unit.Mile : Unit;

            if (_isDrawing)
                GeometryService.CancelSelection();

            _isDrawing = true;

            GeometryService.SelectGeometry(_type, GetGraphic, "Select geometry for measurement.",
                                        GeometryChangeDelegate, unit, SquareUnit);
        }

        private void GetGraphic(DataContracts.Geometry geometry)
        {
            _isDrawing = false;
            GraphicsManager.AddGraphic(geometry, null, null, null);
        }

        private void GeometryChangeDelegate(object sender, GeometryChangedEventArgs e)
        {
            _radius = e.Radius;
            _geometry = e.CurrentGeometry;
            GeometryChange(_geometry, _radius);
        }

        private void GeometryChange(DataContracts.Geometry e, double radiusParameter)
        {
            Unit unit = SquareUnit == SquareUnit.Acre ? Unit.Mile : Unit;
            NotifyOfPropertyChange(() => Unit);

            if (e == null) return;
            if (radiusParameter != 0)
            {
                Radius = Math.Round(UnitConversion.ConvertUnit(_circleStartUnit, unit, radiusParameter), 2);
                Diameter = Math.Round(Radius * 2, 2);
                Circumference = Math.Round(Diameter * Math.PI, 2);
                Area = Math.Round(Math.Pow(radiusParameter, 2) * Math.PI, 2);
                _circleStartUnit = unit;
            }
            if (e.GeometryType.ToUpper() == "ESRIGEOMETRYPOLYGON")
            {
                Area = Math.Round(MeasureManager.GetMeasureManager(unit, SquareUnit).GetPolygonArea(e), 2);
                Perimeter = Math.Round(MeasureManager.GetMeasureManager(unit, SquareUnit).GetPerimeter(e), 2);
            }
            if (e.GeometryType.ToUpper() == "ESRIGEOMETRYENVELOPE")
            {
                DataContracts.Geometry polygon = GeometryTranslator.ConvertEnvelopeToPolygon(e);
                Area = Math.Round(MeasureManager.GetMeasureManager(unit, SquareUnit).GetPolygonArea(polygon), 2);
                Perimeter = Math.Round(MeasureManager.GetMeasureManager(unit, SquareUnit).GetPerimeter(polygon), 2);
            }
            if (e.GeometryType.ToUpper() == "ESRIGEOMETRYPOLYLINE")
            {
                Length = Math.Round(MeasureManager.GetMeasureManager(unit, SquareUnit).GetPolyLineDistance(e), 2);
            }
        }

        public override DependencyObject HelpView
        {
            get
            {
                var help = new MeasureHelpView();
                View.SetModel(help, new MeasureHelpViewModel());
                return help;
            }
        }


        private GenericDialogViewModel _viewModel;
        private MeasureToolView _measureView;

        protected override void OnExecute()
        {
            _graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
            _measureView = new MeasureToolView();
            View.SetModel(_measureView, this);
            //SelectedUnit = Unit.Feet;
            _viewModel = LayoutManager.Instance.ShowPanel(_measureView,
                                                       new DialogServiceArgs
                                                       {
                                                           Title = this.ComponentConfiguration.Name,
                                                           HelpId = "tools;" + GetType().FullName,
                                                           Icon = Icon
                                                       });
            _viewModel.Closed += Dialog_Closed;
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/measure.png"; }
        }

        private void Dialog_Closed(object sender, EventArgs e)
        {

            if (GeometryService != null)
            {
                GeometryService.CancelSelection();
            }

            if (GraphicsManager != null)
            {
                GraphicsManager.ClearGraphics();
                GraphicsManager.Dispose();
            }

            _measureView.DataContext = null;
            _measureView = null;
            _viewModel.Closed -= Dialog_Closed;
            _viewModel = null;
        }
    }
}