﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Components.Views.QueryTool;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Domain.Persistence;
using FDOT.GIS.Client.Domain.Query;
using FDOT.GIS.Client.Query;
using FDOT.GIS.Client.ViewModels;
using FDOT.GIS.DataContracts;
using FDOT.GIS.DataContracts.Enums;

namespace FDOT.GIS.Client.Components.ViewModels.QueryTool
{
    public class QueryCriteriaViewModel : Screen, IDisposable
    {
        private const string COLOR_HEX = "#CC4E9258";

        private readonly GeometrySelectionService _geomtryService =
            new GeometrySelectionService(new GeometrySelectionServiceConfiguration());

        private readonly GraphicsManager _graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
        private GenericDialogViewModel _parentDialog;

        private Layer _layer;
        //private SpatialQueryOptions _options;
        private SaveQueryViewModel _saveViewModel;

        private ObservableCollection<SecondaryQueryCriteria> _secondaryCriterias =
            new ObservableCollection<SecondaryQueryCriteria>();

        private string _selectedGeometryGraphicsId;

        //private DisplayLayer _selectedLayer;
        private QuerySpatialOptionsViewModel _spatialOptionsViewModel;

        public QueryCriteriaViewModel(GenericDialogViewModel parentDialog)
        {
            _parentDialog = parentDialog;
            _parentDialog.Closed += ParentDialogClosed;
            CriterionChangedAction = new Action<CriterionChangedEvent>(CriterionChanged);
            EventAggregatorManager.Events.Subscribe<CriterionChangedEvent>(CriterionChangedAction);
            QuerySettingsViewModel.Instance.PropertyChanged += Instance_PropertyChanged;

            //SetFeatureTypes();
        }

        // need to use an explicit action here so we can unsubscribe it in dispose
        private Action<CriterionChangedEvent> CriterionChangedAction;
        void CriterionChanged(CriterionChangedEvent e)
        { NotifyOfPropertyChange(() => HaveValidQueryOptions); }

        void Instance_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "LimitQueriesToActiveLayers")
            {
                NotifyOfPropertyChange(() => FeatureTypes);
            }
        }

        private FeatureType _selectedFeatureType;
        public FeatureType SelectedFeatureType
        {
            get { return _selectedFeatureType; }
            set
            {
                if (_selectedFeatureType == value) return;
                _selectedFeatureType = value;
                _layer = null;
                SetSpatialOptionsVisiblity();
                NotifyOfPropertyChange(() => SelectedFeatureType);
                NotifyOfPropertyChange(() => IsFeatureTypeSelected);
                SecondaryCriterias.Clear();
            }
        }

        public bool IsFeatureTypeSelected
        {
            get { return SelectedFeatureType != null; }
        }

        public bool SaveQuery { get; set; }

        public QuerySettingsViewModel SettingsViewModel
        {
            get { return QuerySettingsViewModel.Instance; }
        }

        private DataContracts.Geometry _selectedGeometry;
        internal DataContracts.Geometry SelectedGeometry
        {
            get { return _selectedGeometry; }
            set
            {
                _selectedGeometry = value;
                SetSpatialOptionsVisiblity();
                NotifyOfPropertyChange(() => HaveValidQueryOptions);
            }
        }

        public QuerySpatialRelationships? SpatialRelation { get; set; }

        public Layer Layer
        {
            get { return _layer; }
        }

        private bool _isBusy;
        public bool IsBusy
        {
            get { return _isBusy; }
            set
            {
                if (value == _isBusy) return;
                _isBusy = value;
                NotifyOfPropertyChange(() => IsBusy);
            }
        }

        public ObservableCollection<SecondaryQueryCriteria> SecondaryCriterias
        {
            get { return _secondaryCriterias; }
            set { _secondaryCriterias = value; }
        }

        private bool _isSpatialOptionsVisible;
        private Feature _feature;

        public bool IsSpatialOptionsVisible
        {
            get { return _isSpatialOptionsVisible; }
            set
            {
                if (_isSpatialOptionsVisible == value) return;
                _isSpatialOptionsVisible = value;
                NotifyOfPropertyChange(() => IsSpatialOptionsVisible);
            }
        }

        public bool HaveValidQueryOptions
        {
            get
            {
                if (SelectedFeatureType != null && (SelectedGeometry != null ||
                    (SecondaryCriterias != null && SecondaryCriterias.Count > 0 &&
                     SecondaryCriterias.FirstOrDefault(x => x.Attribute != null && !string.IsNullOrEmpty(x.Value)) != null
                    )))
                { return true; }
                return false;
            }
        }

        public List<CriteriaType> AllCriteriaTypes
        {
            get { return new List<CriteriaType> { CriteriaType.All, CriteriaType.Any }; }
        }

        public CriteriaType SelectedCriteriaType { get; set; }
        public DataContracts.Geometry BufferedGeometry { get; set; }

        public IEnumerable<FeatureType> FeatureTypes
        {
            get
            {
                return SettingsViewModel.LimitQueriesToActiveLayers ? MapHandler.ActiveFeatureTypes : MapHandler.AllFeatureTypes;
            }
        }


        private void ParentDialogClosed(object sender, EventArgs e)
        {
            _graphicsManager.Dispose();
            if (_parentDialog != null)
            {
                _parentDialog.Closed -= ParentDialogClosed;
                _parentDialog = null;
            }
            QuerySettingsViewModel.Instance.PropertyChanged -= Instance_PropertyChanged;
            Dispose();
        }

        private void SetSpatialOptionsVisiblity()
        {
            IsSpatialOptionsVisible = SelectedFeatureType != null && SelectedGeometry != null;
        }

        public void Clear()
        {
            _geomtryService.CancelSelection();
            SelectedFeatureType = null;
            SecondaryCriterias.Clear();
            SelectedCriteriaType = CriteriaType.All;
            _graphicsManager.ClearGraphics();
        }

        public void LoadSpatialQueryOptions(QueryParameters options)
        {
            SelectedFeatureType = options.FeatureType;

            SecondaryCriterias.Clear();

            if (options.QueryCriteria != null)
                SecondaryCriterias.Add(new SecondaryQueryCriteria
                                           {
                                               ComparisonType = options.QueryCriteria.ComparisonType,
                                               Attribute = options.QueryCriteria.Attribute,
                                               Value = options.QueryCriteria.Value
                                           });

            if (options.SecondaryCriteria != null && options.SecondaryCriteria.Count() > 0)
            {
                foreach (var secondaryCriteria in options.SecondaryCriteria)
                { SecondaryCriterias.Add(secondaryCriteria); }
            }

            _graphicsManager.ClearGraphics();

            if(options.GeometryCriteria != null)
            {
                SelectedGeometry = options.GeometryCriteria.Geometry;
                SpatialRelation = options.GeometryCriteria.SpatialRelationship;
            }
            
            if (SelectedGeometry != null)
            {
                SymbolManager.SetPalette(COLOR_HEX);
                _graphicsManager.AddGraphic(SelectedGeometry, null, null, null);
                SymbolManager.SetDefaultPalette();
                MapHandler.ZoomToGeometry(SelectedGeometry);
            }

            
        }

        public void AddCriteria()
        {
            SecondaryCriterias.Add(new SecondaryQueryCriteria
                                       {
                                           Operator = (SelectedCriteriaType == CriteriaType.All)
                                                                 ? LogicalOperator.And
                                                                 : LogicalOperator.Or
                                       });
            NotifyOfPropertyChange(() => SecondaryCriterias);
            NotifyOfPropertyChange(() => HaveValidQueryOptions);
        }

        public void DeleteRow(SecondaryQueryCriteria secondaryCriteria)
        {
            SecondaryCriterias.Remove(secondaryCriteria);
            NotifyOfPropertyChange(() => SecondaryCriterias);
            NotifyOfPropertyChange(() => HaveValidQueryOptions);
        }

        public void DrawLine()
        {
            SelectGeometry(GeometryType.Polyline);
        }

        public void DrawBox()
        {
            SelectGeometry(GeometryType.Box);
        }

        public void DrawCircle()
        {
            SelectGeometry(GeometryType.Circle);
        }

        public void DrawPolygon()
        {
            SelectGeometry(GeometryType.Polygon);
        }

        private void SelectGeometry(GeometryType type)
        {
            SymbolManager.SetPalette(COLOR_HEX);
            _geomtryService.CancelSelection();
            SelectedGeometry = null;
            _graphicsManager.ClearGraphics();
            _geomtryService.SelectGeometry(type, GeometrySelected);
        }

        // it's a little muddy putting this view-relevant bit here, but either it goes here or 
        // the control has to be aware of state management
        public void ItemsLoaded(object sender, EventArgs e)
        {
            var ddTree = sender as Controls.DropdownTreeView;
            if (ddTree == null) return;
            if (StateManager.QueriedFeatureTypes != null && StateManager.QueriedFeatureTypes.Count > 0)
            {
                ddTree.AssignItemsToGroup(StateManager.QueriedFeatureTypes, "Recently Queried");
                ddTree.MoveGroupToTop("Recently Queried");
            }
            else
            { ddTree.DefaultGroupName = "All Layers"; }
        }

        // Items in the list aren't intended to be selected and doing so actually causes problems because
        // the items are bound but also mutable, so when re-binding occurs having a selected value results
        // in the list trying to get the selected item by its hash code, which has changed if the object behind
        // the list item has changed.  This is a workaround that simply clears selections as soon as they're made
        // so the list will never have a selection.
        public void CriterionSelected(object sender, RoutedEventArgs e)
        {
            var listBox = sender as System.Windows.Controls.ListBox;
            if (listBox != null)
            {
                if (listBox.SelectionMode == System.Windows.Controls.SelectionMode.Single)
                { listBox.SelectedItem = null; }
                else
                { listBox.SelectedItems.Clear(); }
            }
        }


        public bool RecentItemsEnabled
        {
            get { return StateManager.DoPersistRecentItems; }
        }

        public void ExecuteQuery()
        {
            IsBusy = true;
            if (SaveQuery)
            {
                var view = new SaveQueryView();
                _saveViewModel = new SaveQueryViewModel();
                View.SetModel(view, _saveViewModel);

                GenericDialogViewModel dialog = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs
                                                                                           {
                                                                                               IsModal = true,
                                                                                               Title = "Save Query",
                                                                                               HasMinimizeButton = false,
                                                                                               HasCloseButton = false,
                                                                                               Id = "SaveQuery",
                                                                                               Width = 400,
                                                                                               HasSettingsButton = false
                                                                                           });
                _saveViewModel.Dialog = dialog;
                dialog.Closed += delegate
                                     {
                                         var exContext = new ActionExecutionContext();
                                         Coroutine.BeginExecute(GetResultsBufferWorkflow().GetEnumerator(), exContext, Complete);
                                     };

                return;
            }

            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetResultsBufferWorkflow().GetEnumerator(), executionContext, Complete);
        }

        public void ShareQueryCriteria()
        {
            var view = new QueryShareView();
            var viewModel = new QueryShareViewModel();
            viewModel.QueryParameters = GetQueryparameters();
            View.SetModel(view, viewModel);
            viewModel.Dialog = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs { IsModal = true, Id = "QueryShareView", Height = 270, Width = 400, Title = "Share Query", HasCloseButton = true, HasMinimizeButton = false, HasSettingsButton = false });
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private void GeometrySelected(DataContracts.Geometry geometry)
        {
            SelectedGeometry = geometry;
            if (!string.IsNullOrEmpty(_selectedGeometryGraphicsId))
            { _graphicsManager.RemoveGraphic(_selectedGeometryGraphicsId); }

            _feature = new Feature(null, null, new FeatureEditOptions { CanDelete = false, CanEditGeometry = true }) { Geometry = geometry };
            _feature.GeometryChanged += feature_GeometryChanged;
            _selectedGeometryGraphicsId = _graphicsManager.AddGraphic(_feature, null, null, null);
            SymbolManager.SetDefaultPalette();
        }

        void feature_GeometryChanged(object sender, EventArgs e)
        {
            _selectedGeometry = _feature.Geometry;
        }

        private IEnumerable<IResult> GetResultsBufferWorkflow()
        {
            QueryParameters queryparameters = GetQueryparameters();

            var queryWorkFlow = new ExecuteQuery(queryparameters);
            yield return queryWorkFlow;

            if (_saveViewModel != null && _saveViewModel.SaveQuery)
            {
                StateManager.SaveQuery(_saveViewModel.QueryName, queryparameters);
            }

            StateManager.SaveRecentQuery(queryparameters);

            FeatureType featureType = queryWorkFlow.Result.FeatureType;
            Core.ObservableCollection<Feature> features = queryWorkFlow.Result.Features;

            if (features.Any())
            {
                //var attributeDescription = featureType.Attributes.SingleOrDefault(a => a.IsIdentifier);
                //string label;
                //if (attributeDescription != null)
                //{
                //    label = attributeDescription.Name;
                //                        }
                //else
                //{
                //    label = SelectedFeatureType.Name;
                //}
                    //var args = new ResultDisplayArgs(queryWorkFlow.Result)
                    //               {
                    //                   MapTipColumn = label,
                    //                   MapTipLabel = label,
                    //                   Title = featureType.Name
                    //               };
                    //ResultDisplayManager.Instance.DisplayResults(args);

                var resultset = new FeatureResultSet(queryWorkFlow.Result);
                VirtualApplicationManager.Current.ResultSetManager.AddResultSet(resultset);

                _parentDialog.Close();
                _graphicsManager.Dispose();
            }
            else
            {
                LayoutManager.Instance.ShowMessage("No results found for query", new TimeSpan(0, 0, 0, 8));
            }

            IsBusy = false;
            yield break;
        }

        private QueryParameters GetQueryparameters()
        {
            var queryparameters = new QueryParameters
                                      {
                                          FeatureType = SelectedFeatureType,
                                          GeometryCriteria = new GeometryCriteria
                                                                 {
                                                                     Geometry = BufferedGeometry ?? SelectedGeometry,
                                                                     SpatialRelationship = SpatialRelation
                                                                 }
                                      };


            if(queryparameters.GeometryCriteria.Geometry==null)
            {
                queryparameters.GeometryCriteria = null;
            }

            for (int i = 0; i < SecondaryCriterias.Count; i++)
            {
                SecondaryQueryCriteria criteria = SecondaryCriterias[i];
                if (i == 0)
                {
                    queryparameters.QueryCriteria = criteria;
                }
                else
                {
                    if (criteria.Attribute != null && criteria.Value != null)
                    {
                        criteria.Operator = (SelectedCriteriaType == CriteriaType.All)
                                                ? LogicalOperator.And
                                                : LogicalOperator.Or;
                        queryparameters.SecondaryCriteria.Add(criteria);
                    }
                }
            }
            return queryparameters;
        }

        public void ShowSpatialOptions()
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetLayerWorkflow().GetEnumerator(), executionContext, Complete);
        }

        private IEnumerable<IResult> GetLayerWorkflow()
        {
            //if (_layer == null)
            //{
            //    var layer = new GetLayerWithUrl(SelectedLayer.Id, SelectedLayer.ParentMapService.Url);
            //    yield return layer;
            //    _layer = layer.Result;
            //}

            var view = new QuerySpatialOptionsView();
            _spatialOptionsViewModel = new QuerySpatialOptionsViewModel(SelectedGeometry, SelectedFeatureType, _graphicsManager);

            View.SetModel(view, _spatialOptionsViewModel);

            GenericDialogViewModel dialog = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs
            {
                IsModal = true,
                Title = "Spatial Options",
                HasMinimizeButton = false,
                HasCloseButton = true,
                Id = "SpatialOptions",
                HasSettingsButton = false,
                Width = 450
            });
            dialog.Closed += SpatialOptionsDialogClosed;
            yield break;
        }

        private void SpatialOptionsDialogClosed(object sender, EventArgs e)
        {
            SpatialRelation = _spatialOptionsViewModel.SpatialRelation;
            BufferedGeometry = _spatialOptionsViewModel.BufferedGeometry;
        }

        #region IDisposable implementation

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                EventAggregatorManager.Events.Unsubscribe<CriterionChangedEvent>(CriterionChangedAction);
            }
        }

        // Use C# destructor syntax for finalization code.
        ~QueryCriteriaViewModel()
        {
            // Simply call Dispose(false).
            Dispose(false);
        }

        #endregion
    }

    public class GetLayerWithUrl : ResultBase
    {
        private readonly int _id;
        private readonly string _url;

        #region IResult Members

        public GetLayerWithUrl(int id, string url)
        {
            _id = id;
            _url = url;
        }

        private void OnSvsOnGetLayerWithUrlCompleted(object s, GetLayerWithUrlCompletedEventArgs e)
        {
            Result = e.Result;
            Success();
        }

        public Layer Result { get; set; }
        public override void Execute(ActionExecutionContext context)
        {
            QueryServiceClient svs = ServiceClients.QueryServiceClient;
            svs.GetLayerWithUrlCompleted += OnSvsOnGetLayerWithUrlCompleted;
            svs.GetLayerWithUrlAsync(_id, _url);
        }

        #endregion
    }

    public enum CriteriaType
    {
        All,
        Any
    }
}