﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Caliburn.Micro;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Buffer;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Domain.UnitConversion;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.DataContracts.Enums;
using Buffer = FDOT.GIS.Client.Domain.Buffer.Buffer;

namespace FDOT.GIS.Client.Components.ViewModels.QueryTool
{
    public class QuerySpatialOptionsViewModel : Caliburn.Micro.PropertyChangedBase
    {
        private readonly DataContracts.Geometry _selectedGeometry;
        private readonly FeatureType _featureType;
        private readonly GraphicsManager _graphicsManager;

        public QuerySpatialOptionsViewModel(DataContracts.Geometry selectedGeometry, FeatureType featureType, GraphicsManager graphicsManager)
        {
            _selectedGeometry = selectedGeometry;
            _featureType = featureType;
            _graphicsManager = graphicsManager;
            RefreshSpatialRelations();
        }

        internal DataContracts.Geometry SelectedGeometry
        {
            get { return _selectedGeometry; }
        }

        public GeometrySelectionService GeometryService
        {
            get;
            set;
        }

        private bool _showClearButton;
        public bool ShowClearButton
        {
            get { return _showClearButton; }
            set
            {
                if (_showClearButton == value) return;
                _showClearButton = value;
                NotifyOfPropertyChange(() => ShowClearButton);
            }
        }

        private QuerySpatialRelationships? _spatialRelation = QuerySpatialRelationships.Intersects;

        public QuerySpatialRelationships? SpatialRelation
        {
            get { return _spatialRelation; }
            set
            {
                _spatialRelation = value;
                NotifyOfPropertyChange(() => SpatialRelation);
            }
        }

        public double BufferDistance
        {
            get;
            set;
        }

        public FeatureType FeatureType { get { return _featureType; } }

        public string BufferUnit
        {
            get;
            set;
        }

        public DataContracts.Geometry BufferedGeometry { get; set; }

        public ObservableCollection<string> BufferUnits
        {
            get { return new ObservableCollection<string> { "Feet", "Meter", "Mile", "Kilometer" }; }
        }

        private readonly ObservableCollection<QuerySpatialRelationships> _spatialRelations = new ObservableCollection<QuerySpatialRelationships>();
        private string _bufferedGeometryGraphicsId;

        public ObservableCollection<QuerySpatialRelationships> SpatialRelations
        {
            get { return _spatialRelations; }
        }

        private void RefreshSpatialRelations()
        {
            SpatialRelations.Clear();
            switch (SelectedGeometry.CurrentGeometryType)
            {
                case GeometryTypes.Multipoint:
                case GeometryTypes.Point:
                    switch (FeatureType.GeometryType)
                    {
                        case GeometryTypes.Multipoint:
                        case GeometryTypes.Point:
                            SpatialRelations.Add(QuerySpatialRelationships.Overlaps);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            break;
                        case GeometryTypes.Polyline:
                            SpatialRelations.Add(QuerySpatialRelationships.Touches);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            break;
                        case GeometryTypes.Envelope:
                        case GeometryTypes.Polygon:
                            SpatialRelations.Add(QuerySpatialRelationships.Touches);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            break;
                        default:
                            throw new ArgumentOutOfRangeException();
                    }
                    break;
                case GeometryTypes.Polyline:
                    switch (FeatureType.GeometryType)
                    {
                        case GeometryTypes.Multipoint:
                        case GeometryTypes.Point:
                            SpatialRelations.Add(QuerySpatialRelationships.Overlaps);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            break;
                        case GeometryTypes.Polyline:
                            SpatialRelations.Add(QuerySpatialRelationships.Touches);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            SpatialRelations.Add(QuerySpatialRelationships.Crosses);
                            break;
                        case GeometryTypes.Envelope:
                        case GeometryTypes.Polygon:
                            SpatialRelations.Add(QuerySpatialRelationships.Touches);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            SpatialRelations.Add(QuerySpatialRelationships.Crosses);
                            break;
                        default:
                            throw new ArgumentOutOfRangeException();
                    }
                    break;
                case GeometryTypes.Envelope:
                case GeometryTypes.Polygon:
                    switch (FeatureType.GeometryType)
                    {
                        case GeometryTypes.Multipoint:
                        case GeometryTypes.Point:
                            SpatialRelations.Add(QuerySpatialRelationships.Overlaps);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            break;
                        case GeometryTypes.Polyline:
                            SpatialRelations.Add(QuerySpatialRelationships.Touches);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            SpatialRelations.Add(QuerySpatialRelationships.Crosses);
                            break;
                        case GeometryTypes.Envelope:
                        case GeometryTypes.Polygon:
                            SpatialRelations.Add(QuerySpatialRelationships.Touches);
                            SpatialRelations.Add(QuerySpatialRelationships.Within);
                            SpatialRelations.Add(QuerySpatialRelationships.Contains);
                            SpatialRelations.Add(QuerySpatialRelationships.Intersects);
                            SpatialRelations.Add(QuerySpatialRelationships.Overlaps);
                            break;
                        default:
                            throw new ArgumentOutOfRangeException();
                    }
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
            _spatialRelation = QuerySpatialRelationships.Intersects;
        }

        public void ClearBuffer()
        {
            ShowClearButton = false;
            BufferedGeometry = null;
            _graphicsManager.RemoveGraphic(_bufferedGeometryGraphicsId);
        }

        public IEnumerable<IResult> GetBufferWorkflow()
        {
            var buffer = new Buffer(BufferDistance, BufferUnit, SelectedGeometry);
            yield return buffer;
            BufferedGeometry = buffer.Result;

            if (!string.IsNullOrEmpty(_bufferedGeometryGraphicsId))
            {
                _graphicsManager.RemoveGraphic(_bufferedGeometryGraphicsId);
            }

            _bufferedGeometryGraphicsId = _graphicsManager.AddGraphic(buffer.Result, null, null, null);
            ShowClearButton = true;
        }

        public void ShowBuffer()
        {
            Coroutine.BeginExecute(GetBufferWorkflow().GetEnumerator(), null, BufferComplete);
        }

        private void BufferComplete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw new Exception("Error Buffering", e.Error);
        }
    }
}
