﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Components.Views.Routing;
using FDOT.GIS.Client.Core;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Domain.Routing;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.Routing
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class DrivingDirectionsViewModel : ComponentBase
    {

        public DataContracts.Geometry StartPoint { get; set; }
        public DataContracts.Geometry EndPoint { get; set; }

        private GraphicsManager graphicsManager;

        private string startGraphicId;
        private string endGraphicId;
        private string routeGraphicId;

        public ICommand SelectStartPointCommand { get; set; }
        public ICommand SelectEndPointCommand { get; set; }

        public ICommand ZoomToRouteExtent { get; set; }

        private bool routingInProgress;
        public bool IsRoutingInProgress
        {
            get { return routingInProgress; }
            set
            {
                if (routingInProgress == value) return;

                routingInProgress = value;
                NotifyOfPropertyChange(() => IsRoutingInProgress);
            }
        }

        public void SelectStartPoint()
        {
            var gss = new GeometrySelectionService(new GeometrySelectionServiceConfiguration { Color = Colors.Red, ShowLabels = false });
            gss.SelectGeometry(GeometryType.Point, SetStartPoint);
        }

        public void SelectEndPoint()
        {
            var gss = new GeometrySelectionService(new GeometrySelectionServiceConfiguration { Color = Colors.Red, ShowLabels = false });
            gss.SelectGeometry(GeometryType.Point, SetEndPoint);
        }

        private void SetStartPoint(DataContracts.Geometry geometry)
        {
            if (!string.IsNullOrEmpty(startGraphicId))
            {
                graphicsManager.RemoveGraphic(startGraphicId);
            }
            StartPoint = geometry;
            startGraphicId = graphicsManager.AddGraphic(geometry, new SymbolDefinition("Triangle", Colors.Green), null, null);

            Route();
        }

        private void SetEndPoint(DataContracts.Geometry geometry)
        {
            if (!string.IsNullOrEmpty(endGraphicId))
            {
                graphicsManager.RemoveGraphic(endGraphicId);
            }
            EndPoint = geometry;
            endGraphicId = graphicsManager.AddGraphic(geometry, new SymbolDefinition("Hexagon", Colors.Red), null, null);

            Route();
        }

        private void Route()
        {
            if (StartPoint == null || EndPoint == null) return;

            IsRoutingInProgress = true;
            var messageViewModel = new MessageViewModel { Message = "Calculating Route...", ShowProgress = true };
            LayoutManager.Instance.ShowMessage(messageViewModel, TimeSpan.Zero);

            if (!string.IsNullOrEmpty(routeGraphicId))
                graphicsManager.RemoveGraphic(routeGraphicId);

            Coroutine.BeginExecute(GetWorkflow().GetEnumerator(), null, (sender, args) =>
                                                                            {
                                                                                IsRoutingInProgress = false;
                                                                                LayoutManager.Instance.RemoveMessage(messageViewModel);
                                                                            });
        }

        private ObservableCollection<DataContracts.Geometry> stops;
        private ObservableCollection<DataContracts.Geometry> Stops
        {
            get { return stops; }
            set
            {
                if (stops == value) return;

                stops = value;
                NotifyOfPropertyChange(() => Stops);
            }
        }

        private ICollection<Route> routes;
        public ICollection<Route> Routes
        {
            get { return routes; }
            set
            {
                routes = value;
                NotifyOfPropertyChange(() => Routes);
            }
        }

        private Route selectedRoute;
        public Route SelectedRoute
        {
            get { return selectedRoute; }
            set
            {
                if (selectedRoute == value) return;

                selectedRoute = value;

                if (!string.IsNullOrEmpty(routeGraphicId))
                    graphicsManager.RemoveGraphic(routeGraphicId);
                routeGraphicId = graphicsManager.AddGraphic(selectedRoute.Geometry);

                ZoomToRouteExtent = selectedRoute == null ? null : new DelegateCommand(o =>
                {
                    var zoomCommand = new ZoomToGeometry(selectedRoute.Extent, 1.3);
                    zoomCommand.Execute(null);
                });

                NotifyOfPropertyChange(() => SelectedRoute);
                NotifyOfPropertyChange(() => ZoomToRouteExtent);
            }
        }

        protected override void OnExecute()
        {

            var view = new DrivingDirectionsView();
            view.DataContext = this;

            LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs { Id = "DrivingDirections", Title = "Driving Directions" });
        }

        private IEnumerable<IResult> GetWorkflow()
        {
            IEnumerable<DataContracts.Geometry> stops = new[] { StartPoint, EndPoint };

            if (MapHandler.MapSpatialReference.WellKnownId != WellKnownIds.WGS1984)
            {
                var inSpatialReference = MapHandler.MapSpatialReference;
                var outSpatialReference = new DataContracts.SpatialReference { WellKnownId = WellKnownIds.WGS1984 };

                var project = new Project(inSpatialReference, outSpatialReference, stops);
                yield return project;

                stops = project.Result;
            }


            var route = new SolveRoute(stops);
            yield return route;

            Routes = route.Routes;
            SelectedRoute = Routes.First();

            ZoomToRouteExtent.Execute(null);
        }

        protected override void OnInitialize(IDictionary<string, string> customSettings)
        {
            graphicsManager = GraphicManagerFactory.CreateGraphicsManager();

            SelectStartPointCommand = new DelegateCommand(o => SelectStartPoint());
            SelectEndPointCommand = new DelegateCommand(o => SelectEndPoint());
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/identify.png"; }
        }
    }
}
