﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.ViewModels.Toc;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Components.Views.Toc;
using FDOT.GIS.Client.Core;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class TocViewModel : ComponentBase
    {

        public string BaseMapServiceName
        {
            get { return "Base Maps"; }
        }

        private bool _showExpander;
        public bool ShowExpander
        {
            get { return _showExpander; }
            private set
            {
                _showExpander = MapHandler.BaseMap != null;
                NotifyOfPropertyChange(() => ShowExpander);
            }
        }

        public double BaseMapOpacity
        {
            get { return MapHandler.BaseMap.Opacity; }
            set
            {
                MapHandler.BaseMap.Opacity = value;
                NotifyOfPropertyChange(() => BaseMapOpacity);
            }
        }

        private bool _isBusy;
        public bool IsBusy
        {
            get { return _isBusy; }
            set { _isBusy = value; NotifyOfPropertyChange(() => IsBusy); }
        }

        private string _isBusyMessage;
        public string IsBusyMessage
        {
            get { return _isBusyMessage; }
            set { _isBusyMessage = value; NotifyOfPropertyChange(() => IsBusyMessage); }
        }

        public bool BaseMapVisible
        {
            get { return MapHandler.BaseMap.Visible; }
            set
            {
                MapHandler.BaseMap.Visible = value;
                NotifyOfPropertyChange(() => BaseMapVisible);
            }
        }

        public void BaseMapRadioClicked(object sender, RoutedEventArgs e)
        {
            var radioButton = (RadioButton)sender;
            var mapService = (TiledMapService)radioButton.Tag;
            MapHandler.BaseMap = mapService;
        }

        public MapService SelectedBaseMap
        {
            get { return MapHandler.BaseMap; }
            set
            {
                MapHandler.BaseMap = value;
                NotifyOfPropertyChange(() => SelectedBaseMap);
            }
        }

        private string _searchTerm;
        public string SearchTerm
        {
            get { return _searchTerm; }
            set
            {
                _searchTerm = value;
                NotifyOfPropertyChange(() => SearchTerm);
                UpdateSearchLayers();
            }
        }

        public void UpdateSearchLayers()
        {
            SearchLayers = string.IsNullOrEmpty(SearchTerm) ? new List<DisplayLayer>() : MapHandler.FlattenedLayers.Where(d => d.Name.ToUpper().Contains(SearchTerm.ToUpper()) && d.ParentMapService is DynamicMapService).OrderBy(l => l.Name).ToList();
            NotifyOfPropertyChange(() => SearchLayers);
        }

        private IEnumerable<DisplayLayer> _searchLayers = new List<DisplayLayer>();
        public IEnumerable<DisplayLayer> SearchLayers
        {
            get { return _searchLayers; }
            set { _searchLayers = value; }
        }

        public bool IsSearchVisible
        {
            get { return MapHandler.DynamicLayers != null && MapHandler.DynamicLayers.Any(); }
        }

        public ReadOnlyObservableCollection<TiledMapService> BaseMaps
        {
            get
            {
                return new ReadOnlyObservableCollection<TiledMapService>(new Core.ObservableCollection<TiledMapService>(MapHandler.BaseMaps.OfType<TiledMapService>().Where(l => l.IsViewable)));
            }
        }

        public ReadOnlyObservableCollection<FeatureGraphicsLayer> ClientSideLayers
        {
            get { return MapHandler.DisplayedGraphicsLayers; }
        }

        public ReadOnlyObservableCollection<MapService> MapServices
        {
            get
            {
                return new ReadOnlyObservableCollection<MapService>(new Core.ObservableCollection<MapService>(MapHandler.Layers.OfType<MapService>().Where(l => l.IsViewable &&
                    (l is DynamicMapService || (l is TiledMapService && !((TiledMapService)l).IsBaseMap)))));
            }
        }

        public ICommand LoadShapeFile
        {
            get
            {
                return new DelegateCommand(o =>
                                               {
                                                   var view = new LocalFilesView();
                                                   var viewModel = new LocalFilesViewModel();

                                                   View.SetModel(view, viewModel);
                                                   viewModel.Dialog = LayoutManager.Instance.ShowPanel(view,
                                                                                                       new DialogServiceArgs
                                                                                                           {
                                                                                                               HasCloseButton = true,
                                                                                                               HasMinimizeButton = false,
                                                                                                               HasSettingsButton = false,
                                                                                                               Width = 425,
                                                                                                               IsModal = true,
                                                                                                               Id = "LoadShapeFile",
                                                                                                               Title = "Load Shape File"
                                                                                                           });
                                                   viewModel.Dialog.Closed += delegate
                                                       {
                                                           NotifyOfPropertyChange(() => ClientSideLayers);
                                                       };
                                               });
            }
        }

        public ICommand LoadKml
        {
            get
            {
                return new DelegateCommand(o =>
                                               {
                                                   // Create an instance of the open file dialog box.
                                                   var openFileDialog1 = new OpenFileDialog
                                                                             {
                                                                                 Filter =
                                                                                     "KML/KMZ Files (*.kml, *.kmz)|*.kml;*.kmz",
                                                                                 FilterIndex = 1,
                                                                                 Multiselect = false
                                                                             };

                                                   // Call the ShowDialog method to show the dialog box.
                                                   bool? userClickedOk = openFileDialog1.ShowDialog();

                                                   // Process input if the user clicked OK.
                                                   if (userClickedOk == true)
                                                   {

                                                       Stream fileStream = null;
                                                       try
                                                       {
                                                           // Open the selected file to read.
                                                           fileStream =
                                                               openFileDialog1.File.OpenRead();
                                                           MapHandler.AddKmlLayer(openFileDialog1.File.Name,
                                                                                  fileStream,
                                                                                  KmlLoadingSuccessful,
                                                                                  KmlLoadingUnSuccessful, TocSettingsViewModel.ZoomToKmlOnLoad);
                                                       }
                                                       finally
                                                       {
                                                           if (fileStream != null) fileStream.Close();
                                                       }
                                                   }
                                               });
            }
        }

        public void KmlLoadingSuccessful()
        {
            NotifyOfPropertyChange(() => ClientSideLayers);
            IsBusy = false;
        }

        public void KmlLoadingUnSuccessful()
        {
            LayoutManager.Instance.ShowMessage("The KML/KMZ file specifed could not be loaded.",
                                               new TimeSpan(0, 0, 0, 8));
            IsBusy = false;
        }
        public void TurnOffLayers(object sender, EventArgs e)
        {
            foreach (var l in ((MapService)((FrameworkElement)(sender)).DataContext).Layers)
            {
                l.IsVisible = false;
            }

        }

        public void AddLayer()
        {
            var view = new AddLayerView();
            var viewModel = new AddLayerViewModel();

            View.SetModel(view, viewModel);

            viewModel.Dialog = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs
            {
                IsModal = true,
                Title = "Add Map Service",
                HasMinimizeButton = false,
                HasCloseButton = true,
                HasSettingsButton = false,
                Width = 510,
                Id = "AddLayer"
            });
            viewModel.Dialog.Closed += delegate
                                           {
                                               NotifyOfPropertyChange(() => ClientSideLayers);
                                               NotifyOfPropertyChange(() => MapServices);
                                               NotifyOfPropertyChange(() => BaseMaps);
                                           };

        }

        public bool CanAddLayers
        {
            get
            {
                return ComponentConfiguration.CustomSettings.ContainsKey("CanAddLayers") &&
                       ComponentConfiguration.CustomSettings["CanAddLayers"].ToUpper() == "TRUE"
                           ? true
                           : false;
            }
        }

        #region IComponentBase Members

        protected override string IconResourceUri
        {
            get { return "Assets/Images/toc.png"; }
        }

        public bool ShowAddKml
        {
            get { return TocSettingsViewModel.ShowAddKmlFile; }
        }

        public bool ShowAddMapService
        {
            get { return TocSettingsViewModel.ShowAddMapService; }
        }

        public bool ShowAddShapeFile
        {
            get { return TocSettingsViewModel.ShowAddShapeFile; }
        }

        protected override void OnExecute()
        {
            var tocView = new TocView();
            View.SetModel(tocView, this);
            LayoutManager.Instance.ShowPanel(tocView, new DialogServiceArgs
            {
                Title = this.ComponentConfiguration.Name,
                HelpId = "tools;" + GetType().FullName,
                Icon = this.Icon,
                SettingsViewModel = TocSettingsViewModel,
                ShowSettingsOnBack = true,
                HasSettingsButton = true
            });

            int isBaseMapLoaded = MapHandler.BaseMaps.Count;
            ShowExpander = isBaseMapLoaded >= 1;

            //ShowLegend();
        }

        #endregion

        public TocViewModel()
        {
            TocSettingsViewModel.PropertyChanged += TocSettingsViewModelPropertyChanged;
        }

        void TocSettingsViewModelPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "ShowAddKmlFile")
            {
                NotifyOfPropertyChange(() => ShowAddKml);
            }
            else if (e.PropertyName == "ShowAddMapService")
            {
                NotifyOfPropertyChange(() => ShowAddMapService);
            }
            else if (e.PropertyName == "ShowAddShapeFile")
            {
                NotifyOfPropertyChange(() => ShowAddShapeFile);
            }

        }

        public TocSettingsViewModel TocSettingsViewModel
        {
            get { return TocSettingsViewModel.Instance; }
        }

        public void DeleteRow(MapService mapService)
        {
            MapHandler.RemoveMapService(mapService);
            NotifyOfPropertyChange(() => BaseMaps);
            NotifyOfPropertyChange(() => MapServices);
        }

        public void DeleteClientSideLayer(object sender, EventArgs e)
        {
            var button = (Button)sender;

            MapHandler.RemoveFeatureGraphicsLayer((FeatureGraphicsLayer)button.DataContext);
            button.Click -= DeleteClientSideLayer;

            NotifyOfPropertyChange(() => ClientSideLayers);
        }
        [Hidden]
        public override DependencyObject HelpView
        {
            get
            {
                var help = new TocHelpView();
                View.SetModel(help, new TocHelpViewModel());
                return help;
            }
        }
    }

}
