﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.DotCodes;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class ZoomToViewModel : ComponentBase
    {
        #region Enum
        public enum ZoomTypes
        {
            County,
            District,
            Roadway
        }
        #endregion

        #region Properties

        private readonly ZoomCommandParameters _parameters = new ZoomCommandParameters();

        public static List<string> GetEnumItems
        {
            get
            {
                var zoomList = new List<string>
                                   {
                                       ZoomTypes.County.ToString(),
                                       ZoomTypes.District.ToString(),
                                       ZoomTypes.Roadway.ToString()
                                   };
                return zoomList;
            }
        }

        private string _zoomType;
        public string ZoomType
        {
            get
            {
                if (_zoomType == null)
                {
                    _zoomType = ZoomTypes.County.ToString();
                    return _zoomType;
                }
                return _zoomType;
            }
            set
            {
                // if we're changing zoom types, reset the _parameters
                if (!string.IsNullOrEmpty(value) && !value.Equals(_zoomType, StringComparison.CurrentCultureIgnoreCase))
                { Id = string.Empty; }

                _zoomType = value;
                //Coroutine.BeginExecute(LoadCodesWorkFlow().GetEnumerator());

                switch (_zoomType)
                {
                    case "County":
                        GetLabel = "County Name";
                        break;
                    case "District":
                        GetLabel = "District Id";
                        break;
                    case "Roadway":
                        GetLabel = "Roadway Id";
                        break;
                }
                NotifyOfPropertyChange(() => ZoomType);
                NotifyOfPropertyChange(() => IsCountyCriterion);
                NotifyOfPropertyChange(() => IsDistrictCriterion);
                NotifyOfPropertyChange(() => IsFreeTextCriterion);
            }
        }

        private string _getLabel;
        public string GetLabel
        {
            get
            {
                if (_getLabel == null)
                {
                    _getLabel = "County Name";
                    return _getLabel;
                }
                return _getLabel;
            }
            set
            {
                _getLabel = value;
                NotifyOfPropertyChange(() => GetLabel);
            }
        }

        public string Id
        {
            get { return _parameters.Value; }
            set
            {
                _parameters.Value = value.Trim();
                NotifyOfPropertyChange(() => Id);
            }
        }

        public bool Highlight
        {
            get { return _parameters.Highlight; }
            set 
            { 
                _parameters.Highlight = value;
                NotifyOfPropertyChange(() => Highlight);
            }
        }

        private readonly Dictionary<ZoomTypes, ObservableCollection<DotCode>> _codes = new Dictionary<ZoomTypes, ObservableCollection<DotCode>>();
        public IEnumerable<DotCode> CountyCodes
        {
            get { return _codes.ContainsKey(ZoomTypes.County) ? _codes[ZoomTypes.County] : new ObservableCollection<DotCode>(); }
            set
            {
                // modified to exclude non-geographic counties such as rail, turnpike etc.
                // logic depends on using CONTYDOT DOTCODES as source for counties
                var sortedCollection = new ObservableCollection<DotCode>(value.Where(x => int.Parse(x.Value) < 95).OrderBy(x => x.Description));
                if (_codes.ContainsKey(ZoomTypes.County))
                {
                    _codes[ZoomTypes.County] = sortedCollection;
                }
                else
                { _codes.Add(ZoomTypes.County, sortedCollection); }

                NotifyOfPropertyChange(() => CountyCodes);
            }
        }

        public IEnumerable<DotCode> DistrictCodes
        {
            get { return _codes.ContainsKey(ZoomTypes.District) ? _codes[ZoomTypes.District] : new ObservableCollection<DotCode>(); }
            set
            {
                // modified to exclude non-geographic districts such as statewide and central office
                // logic depends on using DISTRICT DOTCODES as source for counties
                var sortedCollection = new ObservableCollection<DotCode>(value.Where(x =>
                                                                                         { 
                                                                                             int val = int.Parse(x.Value);
                                                                                             return val > 0 && val < 8;
                                                                                         }).OrderBy(x => int.Parse(x.Value)));
                if (_codes.ContainsKey(ZoomTypes.District))
                { _codes[ZoomTypes.District] = new ObservableCollection<DotCode>(sortedCollection); }
                else
                { _codes.Add(ZoomTypes.District, new ObservableCollection<DotCode>(sortedCollection)); }

                NotifyOfPropertyChange(() => DistrictCodes);
            }
        }

        public bool IsFreeTextCriterion
        {
            get { return !(IsCountyCriterion || IsDistrictCriterion); }
        }

        public bool IsCountyCriterion
        {
            get { return ZoomType.Equals(ZoomTypes.County.ToString()); }
        }

        public bool IsDistrictCriterion
        {
            get { return ZoomType.Equals(ZoomTypes.District.ToString()); }
        }

        private ZoomCommand<ZoomCommandParameters> _zoomCommand;

        public void SelectedValueChanged(object sender, EventArgs e)
        {
            var codeDropDown = sender as System.Windows.Controls.ComboBox;
            if (codeDropDown != null)
            {
                _parameters.Value = Convert.ToString(codeDropDown.SelectedValue);
                _parameters.Value = _parameters.Value != null ? _parameters.Value.Trim() : _parameters.Value;
            }
        }

        private IEnumerable<IResult> LoadCodesWorkFlow()
        {
            yield return CodeProviderManager.Instance.GetProvider("CONTYDOT", this, "CountyCodes");
            yield return CodeProviderManager.Instance.GetProvider("DISTRICT", this, "DistrictCodes");
        }
        #endregion

        #region Protected/Public Methods

        protected override void OnInitialize(IDictionary<string, string> customSettings)
        {
            Coroutine.BeginExecute(LoadCodesWorkFlow().GetEnumerator(), null, (s,a) =>
                                                                                  {
                                                                                      if (a.Error != null)
                                                                                          throw a.Error;
                                                                                  });
        }

        protected override void OnExecute()
        {
            var zoomTo = new ZoomToView();
            Highlight = true;
            View.SetModel(zoomTo, this);
            GenericDialogViewModel dialog = LayoutManager.Instance.ShowPanel(zoomTo, new DialogServiceArgs
            {
                Icon = Icon,
                Title = ComponentConfiguration.Name,
                HelpId = "tools;" + GetType().FullName
            });
            dialog.Closed += OnWindowClose;
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/zoom.png"; }
        }

        public void ExecuteZoom()
        {
            if (_zoomCommand != null)
                _zoomCommand.Dispose();

            _zoomCommand = GetZoomCommand(ZoomType);
            _zoomCommand.Execute(_parameters);

        }

        private static ZoomCommand<ZoomCommandParameters> GetZoomCommand(string type)
        {
            switch (type)
            {
                case "County":
                    return (ZoomCommand<ZoomCommandParameters>)VirtualApplicationManager.Current.ScriptableCommands.SingleOrDefault(c => (c as ZoomToCountyCommand) != null);
                case "District":
                    return (ZoomCommand<ZoomCommandParameters>)VirtualApplicationManager.Current.ScriptableCommands.SingleOrDefault(c => (c as ZoomToDistrictCommand) != null);
                case "Roadway":
                    return (ZoomCommand<ZoomCommandParameters>)VirtualApplicationManager.Current.ScriptableCommands.SingleOrDefault(c => (c as ZoomToRoadwayCommand) != null);
                default:
                    throw new ArgumentOutOfRangeException("type");
            }
        }

        private void OnWindowClose(object sender, EventArgs e)
        { _parameters.Highlight = false;
        }


        #endregion

        [Hidden]
        public override System.Windows.DependencyObject HelpView
        {
            get
            {
                var help = new ZoomToolHelpView();
                View.SetModel(help, new ZoomToolHelpViewModel());
                return help;
            }
        }
    }

}
